# Generated by Django 2.2.3 on 2019-11-28 22:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0043_wsedisoapcall'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='wsedisoapcall',
            name='request_identifier',
        ),
        migrations.RemoveField(
            model_name='wsedisoapcall',
            name='timestamp',
        ),
    ]
