# Generated by Django 3.0.4 on 2020-03-25 19:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0059_auto_20200324_0234'),
    ]

    operations = [
        migrations.AddField(
            model_name='referencepaymentbatchfile',
            name='original_filename',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='original filename'),
        ),
        migrations.AddField(
            model_name='statementfile',
            name='original_filename',
            field=models.CharField(blank=True, default='', max_length=256, verbose_name='original filename'),
        ),
    ]
